@@ -1,12 +1,9 @@
+from channels.auth import AuthMiddlewareStack
+from channels.routing import ProtocolTypeRouter, URLRouter
+from django.conf.urls import url
 from otree.channels import consumers
 from otree.extensions import get_extensions_modules
 
-from django.conf.urls import url
-
-from channels.routing import ProtocolTypeRouter, URLRouter, ChannelNameRouter
-from channels.auth import AuthMiddlewareStack
-
-
 websocket_routes = [
     # WebSockets
     url(r'^wait_page/$', consumers.GroupWaitPage),
@@ -32,9 +29,6 @@ websocket_routes = [
         consumers.ChatConsumer,
     ),
     url(r"^export/$", consumers.ExportData),
-    # for django autoreloader
-    # just so client can detect when server has finished restarting
-    url(r'^no_op/$', consumers.NoOp),
 ]
 
 
@@ -44,5 +38,8 @@ for extensions_module in extensions_modules:
 
 
 application = ProtocolTypeRouter(
-    {"websocket": AuthMiddlewareStack(URLRouter(websocket_routes))}
+    {
+        "websocket": AuthMiddlewareStack(URLRouter(websocket_routes)),
+        "lifespan": consumers.LifespanApp,
+    }
 )
